<?php
# This file was generated by 
# http://wiki.bitplan.com/index.php/Pymediawikidocker
# at {{timestamp}}
# 
# https://www.php.net/manual/en/function.phpinfo.php
#  Outputs information about PHP's configuration
phpinfo();


?>
